"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPoSModuleEntry = exports.createGenesisDataObj = exports.createStakersArrayEntry = exports.getStakes = exports.createValidatorsArrayEntry = exports.getValidatorKeys = exports.formatInt = void 0;
const lisk_framework_1 = require("lisk-framework");
const lisk_codec_1 = require("@liskhq/lisk-codec");
const lisk_cryptography_1 = require("@liskhq/lisk-cryptography");
const constants_1 = require("../constants");
const schemas_1 = require("../schemas");
const block_1 = require("../utils/block");
const transaction_1 = require("../utils/transaction");
const ceiling = (a, b) => {
    if (b === 0)
        throw new Error('Can not divide by 0.');
    return Math.floor((a + b - 1) / b);
};
const formatInt = (num) => {
    let buf;
    if (typeof num === 'bigint') {
        if (num < BigInt(0)) {
            throw new Error('Negative number cannot be formatted');
        }
        buf = Buffer.alloc(8);
        buf.writeBigUInt64BE(num);
    }
    else {
        if (num < 0) {
            throw new Error('Negative number cannot be formatted');
        }
        buf = Buffer.alloc(4);
        buf.writeUInt32BE(num, 0);
    }
    return buf.toString('binary');
};
exports.formatInt = formatInt;
const getValidatorKeys = async (accounts, snapshotHeight, snapshotHeightPrevious, db) => {
    const keys = {};
    const blocksStream = db.createReadStream({
        gte: Buffer.from(`${constants_1.DB_KEY_BLOCKS_HEIGHT}:${(0, exports.formatInt)(snapshotHeightPrevious + 1)}`),
        lte: Buffer.from(`${constants_1.DB_KEY_BLOCKS_HEIGHT}:${(0, exports.formatInt)(snapshotHeight)}`),
    });
    const blockIDs = await (0, block_1.getBlocksIDsFromDBStream)(blocksStream);
    for (const id of blockIDs) {
        const blockHeaderBuffer = await db.get(Buffer.from(`${constants_1.DB_KEY_BLOCKS_ID}:${(0, transaction_1.keyString)(id)}`));
        const blockHeader = lisk_codec_1.codec.decode(schemas_1.blockHeaderSchema, blockHeaderBuffer);
        const payload = (await (0, transaction_1.getTransactions)(id, db));
        const base32Address = (0, lisk_cryptography_1.getAddressFromPublicKey)(blockHeader.generatorPublicKey).toString('hex');
        keys[base32Address] = blockHeader.generatorPublicKey.toString('hex');
        for (const trx of payload) {
            const trxSenderAddress = (0, lisk_cryptography_1.getAddressFromPublicKey)(trx.senderPublicKey).toString('hex');
            const account = accounts.find(acc => acc.address.toString('hex') === trxSenderAddress);
            if (account === null || account === void 0 ? void 0 : account.dpos.delegate.username) {
                keys[trxSenderAddress] = trx.senderPublicKey.toString('hex');
            }
        }
    }
    return keys;
};
exports.getValidatorKeys = getValidatorKeys;
const createValidatorsArrayEntry = async (account, validatorKeys, snapshotHeight, tokenID) => {
    if (account.dpos.delegate.username !== constants_1.EMPTY_STRING) {
        const validatorAddress = account.address.toString('hex');
        const validator = Object.freeze({
            address: account.address,
            name: account.dpos.delegate.username,
            blsKey: constants_1.INVALID_BLS_KEY,
            proofOfPossession: constants_1.DUMMY_PROOF_OF_POSSESSION,
            generatorKey: validatorKeys[validatorAddress] || constants_1.INVALID_ED25519_KEY,
            lastGeneratedHeight: account.dpos.delegate.lastForgedHeight,
            isBanned: true,
            reportMisbehaviorHeights: account.dpos.delegate.pomHeights,
            consecutiveMissedBlocks: account.dpos.delegate.consecutiveMissedBlocks,
            lastCommissionIncreaseHeight: snapshotHeight,
            commission: constants_1.MAX_COMMISSION,
            sharingCoefficients: [
                {
                    tokenID,
                    coefficient: constants_1.Q96_ZERO,
                },
            ],
        });
        return validator;
    }
    return null;
};
exports.createValidatorsArrayEntry = createValidatorsArrayEntry;
const getStakes = async (account, tokenID) => {
    const stakes = account.dpos.sentVotes.map(vote => ({
        ...vote,
        sharingCoefficients: [
            {
                tokenID,
                coefficient: constants_1.Q96_ZERO,
            },
        ],
    }));
    const sortedStakes = stakes
        .sort((a, b) => a.delegateAddress.compare(b.delegateAddress))
        .map(({ delegateAddress, ...entry }) => ({
        ...entry,
        validatorAddress: (0, lisk_cryptography_1.getLisk32AddressFromAddress)(delegateAddress),
    }));
    return sortedStakes;
};
exports.getStakes = getStakes;
const createStakersArrayEntry = async (account, tokenID) => {
    if (account.dpos.sentVotes.length || account.dpos.unlocking.length) {
        const staker = {
            address: account.address,
            stakes: await (0, exports.getStakes)(account, tokenID),
            pendingUnlocks: account.dpos.unlocking.map(({ delegateAddress, unvoteHeight, ...unlock }) => ({
                validatorAddress: (0, lisk_cryptography_1.getLisk32AddressFromAddress)(delegateAddress),
                amount: unlock.amount,
                unstakeHeight: unvoteHeight,
            })),
        };
        return staker;
    }
    return null;
};
exports.createStakersArrayEntry = createStakersArrayEntry;
const createGenesisDataObj = async (accounts, delegatesVoteWeights, snapshotHeight) => {
    const r = ceiling(snapshotHeight, constants_1.ROUND_LENGTH);
    const voteWeightR2 = delegatesVoteWeights.voteWeights.find((voteWeight) => voteWeight.round === r - 2);
    if (!voteWeightR2 || voteWeightR2.delegates.length === 0) {
        throw new Error(`Top delegates for round ${r - 2}(r-2)  unavailable, cannot proceed.`);
    }
    const topValidators = voteWeightR2.delegates;
    const initValidators = [];
    const accountbannedMap = new Map(accounts.map(account => [account.address, account.dpos.delegate.isBanned]));
    topValidators.forEach((delegate) => {
        const isAccountBanned = accountbannedMap.get(delegate.address);
        if (!isAccountBanned) {
            initValidators.push(delegate.address);
        }
    });
    const sortedInitValidators = initValidators.sort((a, b) => a.compare(b)).slice(0, 101);
    const genesisDataObj = {
        initRounds: constants_1.POS_INIT_ROUNDS,
        initValidators: sortedInitValidators.map(entry => (0, lisk_cryptography_1.getLisk32AddressFromAddress)(entry)),
    };
    return genesisDataObj;
};
exports.createGenesisDataObj = createGenesisDataObj;
const getPoSModuleEntry = async (validators, stakers, genesisData) => {
    const posObj = {
        validators,
        stakers,
        genesisData,
    };
    return {
        module: constants_1.MODULE_NAME_POS,
        data: posObj,
        schema: lisk_framework_1.posGenesisStoreSchema,
    };
};
exports.getPoSModuleEntry = getPoSModuleEntry;
//# sourceMappingURL=pos.js.map