"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateAsset = void 0;
const lisk_codec_1 = require("@liskhq/lisk-codec");
const lisk_cryptography_1 = require("@liskhq/lisk-cryptography");
const constants_1 = require("./constants");
const schemas_1 = require("./schemas");
const interoperability_1 = require("./assets/interoperability");
const auth_1 = require("./assets/auth");
const token_1 = require("./assets/token");
const pos_1 = require("./assets/pos");
const legacy_1 = require("./assets/legacy");
const block_1 = require("./utils/block");
const AMOUNT_ZERO = BigInt('0');
let totalLSKSupply = AMOUNT_ZERO;
const addressComparator = (a, b) => a.address.compare(b.address);
class CreateAsset {
    constructor(db) {
        this.init = async (snapshotHeight, snapshotHeightPrevious, tokenID) => {
            const authSubstoreEntries = [];
            const userSubstoreEntries = [];
            const supplySubstoreEntries = [];
            const escrowSubstore = [];
            const supportedTokensSubstoreEntries = [];
            const validators = [];
            const stakers = [];
            const encodedUnregisteredAddresses = await this._db.get(Buffer.from(`${constants_1.DB_KEY_CHAIN_STATE}:${constants_1.CHAIN_STATE_UNREGISTERED_ADDRESSES}`));
            const accountStream = await this._db.createReadStream({
                gte: Buffer.from(`${constants_1.DB_KEY_ACCOUNTS_ADDRESS}:${Buffer.alloc(20, 0).toString('binary')}`),
                lte: Buffer.from(`${constants_1.DB_KEY_ACCOUNTS_ADDRESS}:${Buffer.alloc(20, 255).toString('binary')}`),
            });
            const allAccounts = (await (0, block_1.getDataFromDBStream)(accountStream, schemas_1.accountSchema));
            const accounts = allAccounts.filter((acc) => acc.address.length === constants_1.BINARY_ADDRESS_LENGTH);
            const legacyReserveAccount = accounts.find(account => constants_1.ADDRESS_LEGACY_RESERVE.equals(account.address));
            const legacyModuleAssets = await (0, legacy_1.getLegacyModuleEntry)(encodedUnregisteredAddresses, legacyReserveAccount);
            const legacyReserveAmount = (0, legacy_1.getLegacyReserveAmount)();
            const legacyReserve = await (0, token_1.createLegacyReserveAccount)(legacyReserveAccount, legacyReserveAmount, tokenID);
            const validatorKeys = await (0, pos_1.getValidatorKeys)(accounts, snapshotHeight, snapshotHeightPrevious, this._db);
            for (const account of accounts) {
                const authModuleAsset = await (0, auth_1.getAuthModuleEntryBuffer)(account);
                authSubstoreEntries.push(authModuleAsset);
                const userSubstoreEntry = await (0, token_1.createUserSubstoreArrayEntry)(account, tokenID);
                if (userSubstoreEntry)
                    userSubstoreEntries.push(userSubstoreEntry);
                totalLSKSupply += BigInt(account.token.balance);
                const lockedBalances = await (0, token_1.getLockedBalances)(account);
                totalLSKSupply = lockedBalances.reduce((accumulator, lockedBalance) => accumulator + BigInt(lockedBalance.amount), totalLSKSupply);
                const validator = await (0, pos_1.createValidatorsArrayEntry)(account, validatorKeys, snapshotHeight, tokenID);
                if (validator)
                    validators.push(validator);
                const staker = await (0, pos_1.createStakersArrayEntry)(account, tokenID);
                if (staker)
                    stakers.push(staker);
            }
            userSubstoreEntries.push(legacyReserve);
            const sortedUserSubstore = userSubstoreEntries
                .sort((a, b) => a.address.equals(b.address) ? a.tokenID.compare(b.tokenID) : a.address.compare(b.address))
                .map(({ address, ...entry }) => ({
                ...entry,
                address: (0, lisk_cryptography_1.getLisk32AddressFromAddress)(address),
                tokenID: entry.tokenID.toString('hex'),
            }));
            supplySubstoreEntries.push({
                tokenID,
                totalSupply: String(totalLSKSupply + legacyReserveAmount),
            });
            supportedTokensSubstoreEntries.push({
                chainID: constants_1.ALL_SUPPORTED_TOKENS_KEY,
                supportedTokenIDs: [],
            });
            const sortedValidators = validators.sort(addressComparator).map(({ address, ...entry }) => ({
                ...entry,
                address: (0, lisk_cryptography_1.getLisk32AddressFromAddress)(address),
            }));
            const sortedStakers = stakers.sort(addressComparator).map(({ address, ...entry }) => ({
                ...entry,
                address: (0, lisk_cryptography_1.getLisk32AddressFromAddress)(address),
            }));
            const encodedDelegatesVoteWeights = await this._db.get(Buffer.from(`${constants_1.DB_KEY_CHAIN_STATE}:${constants_1.CHAIN_STATE_DELEGATE_VOTE_WEIGHTS}`));
            const decodedDelegatesVoteWeights = await lisk_codec_1.codec.decode(schemas_1.voteWeightsSchema, encodedDelegatesVoteWeights);
            const genesisData = await (0, pos_1.createGenesisDataObj)(accounts, decodedDelegatesVoteWeights, snapshotHeight);
            const sortedAuthSubstoreEntries = authSubstoreEntries
                .sort((a, b) => a.address.compare(b.address))
                .map(entry => ({
                ...entry,
                address: (0, lisk_cryptography_1.getLisk32AddressFromAddress)(entry.address),
            }));
            const authModuleAssets = await (0, auth_1.getAuthModuleEntry)(sortedAuthSubstoreEntries);
            const tokenModuleAssets = await (0, token_1.getTokenModuleEntry)(sortedUserSubstore, supplySubstoreEntries, escrowSubstore, supportedTokensSubstoreEntries);
            const posModuleAssets = await (0, pos_1.getPoSModuleEntry)(sortedValidators, sortedStakers, genesisData);
            const interoperabilityModuleAssets = await (0, interoperability_1.getInteropModuleEntry)();
            const assets = [
                legacyModuleAssets,
                authModuleAssets,
                tokenModuleAssets,
                posModuleAssets,
                interoperabilityModuleAssets,
            ].sort((a, b) => a.module.localeCompare(b.module, 'en'));
            return assets;
        };
        this._db = db;
    }
}
exports.CreateAsset = CreateAsset;
//# sourceMappingURL=createAsset.js.map