"use strict";
const tslib_1 = require("tslib");
const util_1 = tslib_1.__importDefault(require("util"));
const fs = tslib_1.__importStar(require("fs-extra"));
const path_1 = require("path");
const lisk_db_1 = require("@liskhq/lisk-db");
const semver = tslib_1.__importStar(require("semver"));
const command_1 = require("@oclif/command");
const cli_ux_1 = tslib_1.__importDefault(require("cli-ux"));
const constants_1 = require("./constants");
const client_1 = require("./client");
const config_1 = require("./utils/config");
const chain_1 = require("./utils/chain");
const genesis_block_1 = require("./utils/genesis_block");
const createAsset_1 = require("./createAsset");
const node_1 = require("./utils/node");
const fs_1 = require("./utils/fs");
let configCoreV4;
class LiskMigrator extends command_1.Command {
    async run() {
        var _a, _b, _c, _d;
        try {
            const { flags } = this.parse(LiskMigrator);
            const liskCoreV3DataPath = (0, fs_1.resolveAbsolutePath)((_a = flags['lisk-core-v3-data-path']) !== null && _a !== void 0 ? _a : constants_1.DEFAULT_LISK_CORE_PATH);
            const outputPath = (_b = flags.output) !== null && _b !== void 0 ? _b : (0, path_1.join)(__dirname, '..', 'output');
            const snapshotHeight = flags['snapshot-height'];
            const customConfigPath = flags.config;
            const autoMigrateUserConfig = (_c = flags['auto-migrate-config']) !== null && _c !== void 0 ? _c : false;
            const autoStartLiskCoreV4 = flags['auto-start-lisk-core-v4'];
            const snapshotTimeGap = Number((_d = flags['snapshot-time-gap']) !== null && _d !== void 0 ? _d : constants_1.SNAPSHOT_TIME_GAP);
            const client = await (0, client_1.getAPIClient)(liskCoreV3DataPath);
            const nodeInfo = (await client.node.getNodeInfo());
            const { version: appVersion, networkIdentifier } = nodeInfo;
            cli_ux_1.default.action.start('Verifying if backup height from node config matches snapshot height');
            if (snapshotHeight !== nodeInfo.backup.height) {
                this.error(`Snapshot height ${snapshotHeight} does not matches backup height ${nodeInfo.backup.height}.`);
            }
            cli_ux_1.default.action.stop('Snapshot height matches backup height');
            cli_ux_1.default.action.start(`Verifying snapshot height to be multiples of round length i.e ${constants_1.ROUND_LENGTH}`);
            if (snapshotHeight % constants_1.ROUND_LENGTH !== 0) {
                this.error(`Invalid Snapshot Height: ${snapshotHeight}.`);
            }
            cli_ux_1.default.action.stop('Snapshot height is valid');
            const networkConstant = constants_1.NETWORK_CONSTANT[networkIdentifier];
            const outputDir = `${outputPath}/${networkIdentifier}`;
            if (autoStartLiskCoreV4) {
                if (!networkConstant) {
                    this.error(`Unknown network detected. No NETWORK_CONSTANT defined for networkID: ${networkIdentifier}.`);
                }
            }
            cli_ux_1.default.action.start('Verifying Lisk Core version');
            const liskCoreVersion = semver.coerce(appVersion);
            if (!liskCoreVersion) {
                this.error(`Unsupported lisk-core version detected. Supported version range ${constants_1.MIN_SUPPORTED_LISK_CORE_VERSION}.`);
            }
            if (!semver.gte(constants_1.MIN_SUPPORTED_LISK_CORE_VERSION, liskCoreVersion)) {
                this.error(`Lisk Migrator utility is not compatible for lisk-core version ${liskCoreVersion.version}. The minimum compatible version is: ${constants_1.MIN_SUPPORTED_LISK_CORE_VERSION}.`);
            }
            cli_ux_1.default.action.stop(`${liskCoreVersion.version} detected`);
            const configV3 = customConfigPath
                ? await (0, config_1.getConfig)(liskCoreV3DataPath, customConfigPath)
                : await (0, config_1.getConfig)(liskCoreV3DataPath);
            await (0, chain_1.setTokenIDLskByNetID)(networkIdentifier);
            await (0, chain_1.setHeightPrevSnapshotBlockByNetID)(networkIdentifier);
            await (0, chain_1.observeChainHeight)({
                label: 'Waiting for snapshot height to be finalized',
                liskCoreV3DataPath,
                height: snapshotHeight,
                delay: 500,
                isFinal: true,
            });
            cli_ux_1.default.action.start('Creating database instance');
            const snapshotDirPath = (0, path_1.join)(liskCoreV3DataPath, constants_1.SNAPSHOT_DIR);
            const db = new lisk_db_1.Database(snapshotDirPath);
            cli_ux_1.default.action.stop();
            cli_ux_1.default.action.start('Creating genesis assets');
            const createAsset = new createAsset_1.CreateAsset(db);
            const tokenID = (0, chain_1.getTokenIDLsk)();
            const snapshotHeightPrevious = (0, chain_1.getHeightPrevSnapshotBlock)();
            const genesisAssets = await createAsset.init(snapshotHeight, snapshotHeightPrevious, tokenID);
            cli_ux_1.default.action.stop();
            const defaultConfigFilePath = await (0, config_1.resolveConfigPathByNetworkID)(networkIdentifier);
            const defaultConfigV4 = await fs.readJSON(defaultConfigFilePath);
            cli_ux_1.default.action.start(`Exporting genesis block to the path ${outputDir}`);
            await (0, genesis_block_1.writeGenesisAssets)(genesisAssets, outputDir);
            cli_ux_1.default.action.stop();
            if (autoMigrateUserConfig) {
                cli_ux_1.default.action.start('Creating backup for old config');
                await (0, config_1.createBackup)(configV3);
                cli_ux_1.default.action.stop();
                cli_ux_1.default.action.start('Migrating user configuration');
                const migratedConfigV4 = (await (0, config_1.migrateUserConfig)(configV3, defaultConfigV4, snapshotHeight));
                cli_ux_1.default.action.stop();
                cli_ux_1.default.action.start('Validating migrated user configuration');
                const isValidConfig = await (0, config_1.validateConfig)(migratedConfigV4);
                cli_ux_1.default.action.stop();
                if (!isValidConfig)
                    throw new Error('Migrated user configuration is invalid.');
                cli_ux_1.default.action.start(`Exporting user configuration to the path: ${outputDir}`);
                await (0, config_1.writeConfig)(migratedConfigV4, outputDir);
                cli_ux_1.default.action.stop();
                configCoreV4 = migratedConfigV4;
            }
            cli_ux_1.default.action.start('Installing lisk-core v4');
            await (0, node_1.installLiskCore)();
            cli_ux_1.default.action.stop();
            cli_ux_1.default.action.start('Creating genesis block');
            const blockAtSnapshotHeight = (await client.block.getByHeight(snapshotHeight));
            await (0, genesis_block_1.createGenesisBlock)(networkConstant.name, defaultConfigFilePath, outputDir, blockAtSnapshotHeight, snapshotTimeGap);
            cli_ux_1.default.action.stop();
            if (autoStartLiskCoreV4) {
                try {
                    if (!autoMigrateUserConfig) {
                        configCoreV4 = defaultConfigV4;
                    }
                    cli_ux_1.default.action.start(`Creating legacy.db at ${constants_1.LEGACY_DB_PATH}`);
                    await (0, fs_1.copyDir)(snapshotDirPath, (0, fs_1.resolveAbsolutePath)(constants_1.LEGACY_DB_PATH));
                    this.log(`Legacy database has been created at ${constants_1.LEGACY_DB_PATH}`);
                    cli_ux_1.default.action.stop();
                    const isLiskCoreV3Stopped = await cli_ux_1.default.confirm(`
					Please stop Lisk Core v3 to continue. Type 'yes' and press Enter when ready. [yes/no]`);
                    if (isLiskCoreV3Stopped) {
                        const isUserConfirmed = await cli_ux_1.default.confirm(`
						Start Lisk Core with the following configuration? [yes/no] \n
						${util_1.default.inspect(configCoreV4, false, 3)}`);
                        if (isUserConfirmed) {
                            cli_ux_1.default.action.start('Starting lisk-core v4');
                            const network = networkConstant.name;
                            await (0, node_1.startLiskCore)(this, liskCoreV3DataPath, configCoreV4, network, outputDir);
                            this.log('Started Lisk Core v4 at default data directory.');
                            cli_ux_1.default.action.stop();
                        }
                        else {
                            this.log('User did not accept the migrated config. Skipping the Lisk Core v4 auto-start process.');
                        }
                    }
                    else {
                        this.log('User did not confirm Lisk Core v3 node shutdown. Skipping the Lisk Core v4 auto-start process.');
                    }
                }
                catch (err) {
                    this.error(`Failed to auto-start Lisk Core v4.\nError: ${err}`);
                }
            }
            else {
                this.log(`Please copy ${snapshotDirPath} directory to the Lisk Core V4 data directory in order to access legacy blockchain information`);
            }
        }
        catch (error) {
            this.error(error);
        }
        this.log('Successfully finished migration. Exiting!!!');
        process.exit(0);
    }
}
LiskMigrator.description = 'Migrate Lisk Core to latest version';
LiskMigrator.flags = {
    version: command_1.flags.version({ char: 'v' }),
    help: command_1.flags.help({ char: 'h' }),
    output: command_1.flags.string({
        char: 'o',
        required: false,
        description: 'File path to write the genesis block json. If not provided, it will default to cwd/genesis_block.json.',
    }),
    'lisk-core-v3-data-path': command_1.flags.string({
        char: 'd',
        required: false,
        description: 'Path where the lisk-core v3.x instance is running. The current home directory will be considered the default directory if not otherwise provided.',
    }),
    config: command_1.flags.string({
        char: 'c',
        required: false,
        description: 'Custom configuration file path.',
    }),
    'snapshot-height': command_1.flags.integer({
        char: 's',
        required: true,
        env: 'SNAPSHOT_HEIGHT',
        description: 'The height at which re-genesis block will be generated. Can be specified with SNAPSHOT_HEIGHT as well.',
    }),
    'snapshot-time-gap': command_1.flags.integer({
        required: false,
        env: 'SNAPSHOT_TIME_GAP',
        description: 'The number of seconds elapsed between the block at height HEIGHT_SNAPSHOT and the snapshot block.',
    }),
    'auto-migrate-config': command_1.flags.boolean({
        required: false,
        env: 'AUTO_MIGRATE_CONFIG',
        description: 'Migrate user configuration automatically. Default to false.',
        default: false,
    }),
    'auto-start-lisk-core-v4': command_1.flags.boolean({
        required: false,
        env: 'AUTO_START_LISK_CORE',
        description: 'Start lisk core v4 automatically. Default to false.',
        default: false,
    }),
};
module.exports = LiskMigrator;
//# sourceMappingURL=index.js.map